@@ -1,24 +1,23 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007,2008 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
 
 package IO::Async::ChildManager;
 
 use strict;
+use warnings;
 
-our $VERSION = '0.16.001';
+our $VERSION = '0.46_001';
 
 # Not a notifier
 
 use IO::Async::Stream;
-use IO::Async::MergePoint;
 
 use Carp;
 use Scalar::Util qw( weaken );
 
-use Fcntl qw( F_GETFL F_SETFL FD_CLOEXEC );
-use POSIX qw( WNOHANG _exit sysconf _SC_OPEN_MAX dup2 nice );
+use POSIX qw( _exit sysconf _SC_OPEN_MAX dup2 nice );
 
 use constant LENGTH_OF_I => length( pack( "I", 0 ) );
 
@@ -35,29 +34,133 @@ C<IO::Async::ChildManager> - facilitates the execution of child processes
 This object is used indirectly via an C<IO::Async::Loop>:
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ use POSIX qw( WEXITSTATUS );
+
+ my $loop = IO::Async::Loop->new;
 
  ...
 
- $loop->watch_child( 1234 => sub { print "Child 1234 exited\n" } );
+ $loop->run_child(
+    command => "/bin/ps",
+
+    on_finish => sub {
+       my ( $pid, $exitcode, $stdout, $stderr ) = @_;
+       my $status = WEXITSTATUS( $exitcode );
+       print "ps [PID $pid] exited with status $status\n";
+    },
+ );
+
+ $loop->open_child(
+    command => [ "/bin/ping", "-c4", "some.host" ],
+
+    stdout => {
+       on_read => sub {
+          my ( $stream, $buffref, $eof ) = @_;
+          while( $$buffref =~ s/^(.*)\n// ) {
+             print "PING wrote: $1\n";
+          }
+          return 0;
+       },
+    },
+
+    on_finish => sub {
+       my ( $pid, $exitcode ) = @_;
+       my $status = WEXITSTATUS( $exitcode );
+       ...
+    },
+ );
+
+ my ( $pipeRd, $pipeWr ) = $loop->pipepair;
+ $loop->spawn_child(
+    command => "/usr/bin/my-command",
+
+    setup => [
+       stdin  => [ "open", "<", "/dev/null" ],
+       stdout => $pipeWr,
+       stderr => [ "open", ">>", "/var/log/mycmd.log" ],
+       chdir  => "/",
+    ]
+
+    on_exit => sub {
+       my ( $pid, $exitcode ) = @_;
+       my $status = WEXITSTATUS( $exitcode );
+       print "Command exited with status $status\n";
+    },
+ );
 
  $loop->spawn_child(
-    command => "/usr/bin/something",
-    on_exit => \&exit_handler,
-    setup => [
-       stdout => $pipe,
-    ]
+    code => sub {
+       do_something; # executes in a child process
+       return 1;
+    },
+
+    on_exit => sub {
+       my ( $pid, $exitcode, $dollarbang, $dollarat ) = @_;
+       my $status = WEXITSTATUS( $exitcode );
+       print "Child process exited with status $status\n";
+       print " OS error was $dollarbang, exception was $dollarat\n";
+    },
  );
 
 =head1 DESCRIPTION
 
-This module extends the functionallity of the containing C<IO::Async::Loop> to
+This module extends the functionality of the containing C<IO::Async::Loop> to
 manage the execution of child processes. It acts as a central point to store
-PID values of currently-running children, and to call the appropriate callback
-handler code when the process terminates.
+PID values of currently-running children, and to call the appropriate
+continuation handler code when the process terminates. It provides useful
+wrapper methods that set up filehandles and other child process details, and
+to capture the child process's STDOUT and STDERR streams.
 
 =cut
 
+# Writing to variables of $> and $) have tricky ways to obtain error results
+sub setuid
+{
+   my ( $uid ) = @_;
+
+   $> = $uid; my $saved_errno = $!;
+   $> == $uid and return 1;
+
+   $! = $saved_errno;
+   return undef;
+}
+
+sub setgid
+{
+   my ( $gid ) = @_;
+
+   $) = $gid; my $saved_errno = $!;
+   $) == $gid and return 1;
+
+   $! = $saved_errno;
+   return undef;
+}
+
+sub setgroups
+{
+   my @groups = @_;
+
+   my $gid = $)+0;
+   # Put the primary GID as the first group in the supplementary list, because
+   # some operating systems ignore this position, expecting it to indeed be
+   # the primary GID.
+   # See
+   #   https://rt.cpan.org/Ticket/Display.html?id=65127
+   @groups = grep { $_ != $gid } @groups;
+
+   $) = "$gid $gid " . join " ", @groups; my $saved_errno = $!;
+
+   # No easy way to detect success or failure. Just check that we have all and
+   # only the right groups
+   my %gotgroups = map { $_ => 1 } split ' ', "$)";
+
+   $! = $saved_errno;
+   $gotgroups{$_}-- or return undef for @groups;
+   keys %gotgroups or return undef;
+
+   return 1;
+}
+
 # Internal constructor
 sub new
 {
@@ -67,26 +170,14 @@ sub new
    my $loop = delete $params{loop} or croak "Expected a 'loop'";
 
    my $self = bless {
-      childdeathhandlers => {},
       loop => $loop,
    }, $class;
 
    weaken( $self->{loop} );
 
-   $loop->attach_signal( CHLD => sub { $self->SIGCHLD } );
-
    return $self;
 }
 
-sub disable
-{
-   my $self = shift;
-
-   my $loop = $self->{loop};
-
-   $loop->detach_signal( 'CHLD' );
-}
-
 =head1 METHODS
 
 When active, the following methods are available on the containing C<Loop>
@@ -94,175 +185,6 @@ object.
 
 =cut
 
-sub SIGCHLD
-{
-   my $self = shift;
-
-   my $handlermap = $self->{childdeathhandlers};
-
-   my $count = 0;
-
-   while( 1 ) {
-      my $zid = waitpid( -1, WNOHANG );
-
-      last if !defined $zid or $zid < 1;
- 
-      if( defined $handlermap->{$zid} ) {
-         $handlermap->{$zid}->( $zid, $? );
-         delete $handlermap->{$zid};
-      }
-      else {
-         carp "No child death handler for '$zid'";
-      }
-
-      $count++;
-   }
-
-   return $count;
-}
-
-=head2 $loop->watch_child( $kid, $code )
-
-This method adds a new handler for the termination of the given child PID.
-
-=over 8
-
-=item $kid
-
-The PID to watch.
-
-=item $code
-
-A CODE reference to the handling function. It will be invoked as
-
- $code->( $pid, $? )
-
-After invocation, the handler is automatically removed from the manager.
-
-=back
-
-=cut
-
-sub watch_child
-{
-   my $self = shift;
-   my ( $kid, $code ) = @_;
-
-   my $handlermap = $self->{childdeathhandlers};
-
-   croak "Already have a handler for $kid" if exists $handlermap->{$kid};
-   $handlermap->{$kid} = $code;
-
-   return;
-}
-
-=head2 $watching = $manager->is_watching( $kid )
-
-This method tests if the manager is currently watching for termination of the
-given PID. It returns a boolean value.
-
-=over 8
-
-=item $kid
-
-The PID.
-
-=back
-
-=cut
-
-sub is_watching
-{
-   my $self = shift;
-   my ( $kid ) = @_;
-
-   my $handlermap = $self->{childdeathhandlers};
-
-   return exists $handlermap->{$kid};
-}
-
-=head2 @kids = $manager->list_watching()
-
-This method returns a list of the PIDs that the manager is currently watching
-for. The list is returned in no particular order.
-
-=cut
-
-sub list_watching
-{
-   my $self = shift;
-
-   my $handlermap = $self->{childdeathhandlers};
-
-   return keys %$handlermap;
-}
-
-=head2 $pid = $loop->detach_child( %params )
-
-This method creates a new child process to run a given code block.
-
-=over 8
-
-=item code => CODE
-
-A block of code to execute in the child process. It will be called in scalar
-context inside an C<eval> block. The return value will be used as the
-C<exit()> code from the child if it returns (or 255 if it returned C<undef> or
-thows an exception).
-
-=item on_exit => CODE
-
-A optional callback function to be called when the child processes exits. It
-will be invoked in the following way:
-
- $on_exit->( $pid, $exitcode )
-
-This key is optional; if not supplied, the calling code should install a
-handler using the C<watch_child()> method.
-
-=item keep_signals => BOOL
-
-Optional boolean. If missing or false, any CODE references in the C<%SIG> hash
-will be removed and restored back to C<DEFAULT> in the child process. If true,
-no adjustment of the C<%SIG> hash will be performed.
-
-=back
-
-=cut
-
-sub detach_child
-{
-   my $self = shift;
-   my %params = @_;
-
-   my $code = $params{code};
-
-   my $kid = fork();
-   defined $kid or croak "Cannot fork() - $!";
-
-   if( $kid == 0 ) {
-      unless( $params{keep_signals} ) {
-         foreach( keys %SIG ) {
-            next if m/^__(WARN|DIE)__$/;
-            $SIG{$_} = "DEFAULT" if ref $SIG{$_} eq "CODE";
-         }
-      }
-
-      my $exitvalue = eval { $code->() };
-
-      defined $exitvalue or $exitvalue = -1;
-      _exit( $exitvalue );
-   }
-
-   my $loop = $self->{loop};
-
-   if( defined $params{on_exit} ) {
-      $loop->watch_child( $kid => $params{on_exit} );
-   }
-
-   return $kid;
-}
-
 =head2 $pid = $loop->spawn_child( %params )
 
 This method creates a new child process to run a given code block or command.
@@ -274,7 +196,7 @@ The C<%params> hash takes the following keys:
 
 Either a reference to an array containing the command and its arguments, or a
 plain string containing the command. This value is passed into perl's
-C<exec()> function.
+C<exec> function.
 
 =item code => CODE
 
@@ -288,32 +210,36 @@ process before running the code or command. See below.
 
 =item on_exit => CODE
 
-A callback function to be called when the child processes exits. It will be
-invoked in the following way:
+A continuation to be called when the child processes exits. It will be invoked
+in the following way:
 
  $on_exit->( $pid, $exitcode, $dollarbang, $dollarat )
 
+The second argument is passed the plain perl C<$?> value. To use that
+usefully, see C<WEXITSTATUS> and others from C<POSIX>.
+
 =back
 
 Exactly one of the C<command> or C<code> keys must be specified.
 
 If the C<command> key is used, the given array or string is executed using the
-C<exec()> function. 
+C<exec> function. 
 
-If the C<code> key is used, the return value will be used as the C<exit()>
+If the C<code> key is used, the return value will be used as the C<exit(2)>
 code from the child if it returns (or 255 if it returned C<undef> or thows an
 exception).
 
- Case            | WEXITSTATUS($exitcode) | $dollarbang | $dollarat
- ----------------+------------------------+-------------+----------
- exec() succeeds | exit code from program |     0       |    ""
- exec() fails    |         255            |     $!      |    ""
- $code returns   |     return value       |     $!      |    ""
- $code dies      |         255            |     $!      |    $@
+ Case          | WEXITSTATUS($exitcode) | $dollarbang | $dollarat
+ --------------+------------------------+-------------+----------
+ exec succeeds | exit code from program |     0       |    ""
+ exec fails    |         255            |     $!      |    ""
+ $code returns |     return value       |     $!      |    ""
+ $code dies    |         255            |     $!      |    $@
 
-It is usually more convenient to use the C<open()> method in simple cases
+It is usually more convenient to use the C<open_child> method in simple cases
 where an external program is being started in order to interact with it via
-file IO.
+file IO, or even C<run_child> when only the final result is required, rather
+than interaction while it is running.
 
 =cut
 
@@ -341,12 +267,15 @@ sub spawn_child
 
    my $loop = $self->{loop};
 
-   my ( $readpipe, $writepipe ) = $loop->pipepair() or croak "Cannot pipe() - $!";
+   my ( $readpipe, $writepipe );
 
-   my $flags = fcntl( $writepipe, F_GETFL, 0 ) or 
-      croak "Cannot fcntl(F_GETFL) - $!";
-   fcntl( $writepipe, F_SETFL, $flags | FD_CLOEXEC ) or
-      croak "Cannot fcntl(F_SETFL) - $!";
+   {
+      # Ensure it's FD_CLOEXEC - this is a bit more portable than manually
+      # fiddling with F_GETFL and F_SETFL (e.g. MSWin32)
+      local $^F = -1;
+
+      ( $readpipe, $writepipe ) = $loop->pipepair or croak "Cannot pipe() - $!";
+   }
 
    if( defined $command ) {
       my @command = ref( $command ) ? @$command : ( $command );
@@ -358,7 +287,7 @@ sub spawn_child
       };
    }
 
-   my $kid = $loop->detach_child( 
+   my $kid = $loop->fork( 
       code => sub {
          # Child
          close( $readpipe );
@@ -374,7 +303,7 @@ sub spawn_child
 =head2 C<setup> array
 
 This array gives a list of file descriptor operations to perform in the child
-process after it has been C<fork()>ed from the parent, before running the code
+process after it has been C<fork(2)>ed from the parent, before running the code
 or command. It consists of name/value pairs which are ordered; the operations
 are performed in the order given.
 
@@ -393,12 +322,12 @@ The file descriptor will be closed.
 
 =item [ 'dup', $io ]
 
-The file descriptor will be C<dup2()>ed from the given IO handle.
+The file descriptor will be C<dup2(2)>ed from the given IO handle.
 
 =item [ 'open', $mode, $file ]
 
 The file descriptor will be opened from the named file in the given mode. The
-C<$mode> string should be in the form usually given to the C<open()> function;
+C<$mode> string should be in the form usually given to the C<open> function;
 such as '<' or '>>'.
 
 =item [ 'keep' ]
@@ -437,11 +366,30 @@ A reference to a hash to set as the child process's environment.
 
 =item nice => INT
 
-Change the child process's scheduling priority using C<POSIX::nice()>.
+Change the child process's scheduling priority using C<POSIX::nice>.
 
 =item chdir => STRING
 
-Change the child process's working directory using C<chdir()>.
+Change the child process's working directory using C<chdir>.
+
+=item setuid => INT
+
+=item setgid => INT
+
+Change the child process's effective UID or GID.
+
+=item setgroups => ARRAY
+
+Change the child process's groups list, to those groups whose numbers are
+given in the ARRAY reference.
+
+On most systems, only the privileged superuser change user or group IDs.
+C<IO::Async> will B<NOT> check before detaching the child process whether
+this is the case.
+
+If setting both the primary GID and the supplementary groups list, it is
+suggested to set the primary GID first. Moreover, some operating systems may
+require that the supplementary groups list contains the primary GID.
 
 =back
 
@@ -449,6 +397,10 @@ If no directions for what to do with C<stdin>, C<stdout> and C<stderr> are
 given, a default of C<keep> is implied. All other file descriptors will be
 closed, unless a C<keep> operation is given for them.
 
+If C<setuid> is used, be sure to place it after any other operations that
+might require superuser privileges, such as C<setgid> or opening special
+files.
+
 =cut
 
 sub _check_setup_and_canonicise
@@ -462,6 +414,8 @@ sub _check_setup_and_canonicise
 
    my @setup;
 
+   my $has_setgroups;
+
    foreach my $i ( 0 .. $#$setup / 2 ) {
       my ( $key, $value ) = @$setup[$i*2, $i*2 + 1];
 
@@ -483,7 +437,7 @@ sub _check_setup_and_canonicise
          elsif( $ref eq "ARRAY" ) {
             # Already OK
          }
-         elsif( $ref eq "GLOB" ) {
+         elsif( $ref eq "GLOB" or eval { $value->isa( "IO::Handle" ) } ) {
             $value = [ 'dup', $value ];
          }
          else {
@@ -506,6 +460,18 @@ sub _check_setup_and_canonicise
          # ARRAY(0x12345) are unlikely to exist
          -d $value or croak "Working directory '$value' does not exist";
       }
+      elsif( $key eq "setuid" ) {
+         $value =~ m/^\d+$/ or croak "Expected integer for 'setuid' setup key";
+      }
+      elsif( $key eq "setgid" ) {
+         $value =~ m/^\d+$/ or croak "Expected integer for 'setgid' setup key";
+         $has_setgroups and carp "It is suggested to 'setgid' before 'setgroups'";
+      }
+      elsif( $key eq "setgroups" ) {
+         ref $value eq "ARRAY" or croak "Expected ARRAY reference for 'setgroups' setup key";
+         m/^\d+$/ or croak "Expected integer in 'setgroups' array" for @$value;
+         $has_setgroups = 1;
+      }
       else {
          croak "Unrecognised setup operation '$key'";
       }
@@ -523,7 +489,7 @@ sub _spawn_in_parent
 
    my $loop = $self->{loop};
 
-   # We need to wait for both the errno pipe to close, and for waitpid()
+   # We need to wait for both the errno pipe to close, and for waitpid
    # to give us an exit code. We'll form two closures over these two
    # variables so we can cope with those happening in either order
 
@@ -533,10 +499,12 @@ sub _spawn_in_parent
    my $pipeclosed = 0;
 
    $loop->add( IO::Async::Stream->new(
+      notifier_name => "statuspipe,kid=$kid",
+
       read_handle => $readpipe,
 
       on_read => sub {
-         my ( $self, $buffref, $closed ) = @_;
+         my ( $self, $buffref, $eof ) = @_;
 
          if( !defined $dollarbang ) {
             if( length( $$buffref ) >= 2 * LENGTH_OF_I ) {
@@ -552,7 +520,7 @@ sub _spawn_in_parent
             }
          }
 
-         if( $closed ) {
+         if( $eof ) {
             $dollarbang = 0  if !defined $dollarbang;
             if( !defined $length_dollarat ) {
                $length_dollarat = 0;
@@ -600,7 +568,7 @@ sub _spawn_in_child
 
       if( @$setup ) {
          # The writepipe might be in the way of a setup filedescriptor. If it
-         # is we'll have to dup2() it out of the way then close the original.
+         # is we'll have to dup2 it out of the way then close the original.
          foreach my $i ( 0 .. $#$setup/2 ) {
             my ( $key, $value ) = @$setup[$i*2, $i*2 + 1];
             $key =~ m/^fd(\d+)$/ or next;
@@ -620,7 +588,7 @@ sub _spawn_in_child
                $fd_in_use{$fd} = 1;
 
                my $fileno = fileno $params[0];
-               # Keep a count of how many times it will be dup()ed from so we
+               # Keep a count of how many times it will be dup'ed from so we
                # can close it once we've finished
                $fds_refcount{$fileno}++;
             };
@@ -686,6 +654,15 @@ sub _spawn_in_child
             elsif( $key eq "chdir" ) {
                chdir( $value ) or die "Cannot chdir('$value') - $!";
             }
+            elsif( $key eq "setuid" ) {
+               setuid( $value ) or die "Cannot setuid('$value') - $!";
+            }
+            elsif( $key eq "setgid" ) {
+               setgid( $value ) or die "Cannot setgid('$value') - $!";
+            }
+            elsif( $key eq "setgroups" ) {
+               setgroups( @$value ) or die "Cannot setgroups() - $!";
+            }
          }
       }
 
@@ -701,328 +678,10 @@ sub _spawn_in_child
    return $exitvalue;
 }
 
-=head2 $pid = $loop->open_child( %params )
-
-This creates a new child process to run the given code block or command, and
-attaches filehandles to it that the parent will watch. The C<%params> hash
-takes the following keys:
-
-=over 8
-
-=item command => ARRAY or STRING
-
-=item code => CODE
-
-The command or code to run in the child process (as per the C<spawn> method)
-
-=item on_finish => CODE
-
-A callback function to be called when the child process exits and has closed
-all of the filehandles that were set up for it. It will be invoked in the
-following way:
-
- $on_finish->( $pid, $exitcode )
-
-=item on_error => CODE
-
-Optional callback to be called when the child code block throws an exception,
-or the command could not be C<exec()>ed. It will be invoked in the following
-way (as per C<spawn>)
-
- $on_error->( $pid, $exitcode, $dollarbang, $dollarat )
-
-If this callback is not supplied, then C<on_finish> is used instead. The value
-of C<$!> and C<$@> will not be reported.
-
-=item setup => ARRAY
-
-Optional reference to an array to pass to the underlying C<spawn> method.
-
-=back
-
-In addition, the hash takes keys that define how to set up file descriptors in
-the child process. (If the C<setup> array is also given, these operations will
-be performed after those specified by C<setup>.)
-
-=over 8
-
-=item fdI<n> => HASH
-
-A hash describing how to set up file descriptor I<n>. The hash may contain one
-of the following sets of keys:
-
-=over 4
-
-=item on_read => CODE
-
-The child will be given the writing end of a pipe. The reading end will be
-wrapped by an C<IO::Async::Stream> using this C<on_read> callback function.
-
-=item from => STRING
-
-The child will be given the reading end of a pipe. The string given by the
-C<from> parameter will be written to the child. When all of the data has been
-written the pipe will be closed.
-
-=back
-
-=item stdin => ...
-
-=item stdout => ...
-
-=item stderr => ...
-
-Shortcuts for C<fd0>, C<fd1> and C<fd2> respectively.
-
-=back
-
-=cut
-
-sub open_child
-{
-   my $self = shift;
-   my %params = @_;
-
-   my %subparams;
-   my @setup;
-   my %filehandles;
-
-   my $on_finish = delete $params{on_finish};
-   ref $on_finish eq "CODE" or croak "Expected 'on_finish' to be a CODE ref";
-
-   my $on_error = delete $params{on_error};
-   if( $on_error ) {
-      ref $on_error eq "CODE" or croak "Expected 'on_error' to be a CODE ref";
-   }
-
-   $params{on_exit} and croak "Cannot pass 'on_exit' parameter through ChildManager->open";
-
-   if( $params{setup} ) {
-      ref $params{setup} eq "ARRAY" or croak "Expected 'setup' to be an ARRAY ref";
-      @setup = @{ $params{setup} };
-      delete $params{setup};
-   }
-
-   my $loop = $self->{loop};
-
-   foreach my $key ( keys %params ) {
-      my $value = $params{$key};
-
-      my $orig_key = $key;
-
-      # Rewrite stdin/stdout/stderr
-      $key eq "stdin"  and $key = "fd0";
-      $key eq "stdout" and $key = "fd1";
-      $key eq "stderr" and $key = "fd2";
-
-      if( $key =~  m/^fd\d+$/ ) {
-         ref $value eq "HASH" or croak "Expected '$orig_key' to be a HASH ref";
-
-         my ( $pipe_r, $pipe_w ) = $loop->pipepair() or croak "Unable to pipe() - $!";
-
-         my ( $myfd, $childfd );
-
-         if( exists $value->{on_read} ) {
-            ref $value->{on_read} eq "CODE" or croak "Expected 'on_read' for '$orig_key' be a CODE ref";
-            scalar keys %$value == 1 or croak "Found other keys than 'on_read' for '$orig_key'";
-
-            $myfd    = $pipe_r;
-            $childfd = $pipe_w;
-         }
-         elsif( exists $value->{from} ) {
-            ref $value->{from} eq "" or croak "Expected 'from' for '$orig_key' not to be a reference";
-            scalar keys %$value == 1 or croak "Found other keys than 'from' for '$orig_key'";
-
-            $myfd    = $pipe_w;
-            $childfd = $pipe_r;
-         }
-         else {
-            croak "Cannot recognise what to do with '$orig_key'";
-         }
-
-         $filehandles{$key} = [ $myfd, $childfd, $value ];
-         push @setup, $key => [ dup => $childfd ];
-      }
-      else {
-         $subparams{$orig_key} = $value;
-      }
-   }
-
-   my $pid;
-
-   my $mergepoint = IO::Async::MergePoint->new(
-      needs => [ "exit", keys %filehandles ],
-
-      on_finished => sub {
-         my %items = @_;
-         my ( $exitcode, $dollarbang, $dollarat ) = @{ $items{exit} };
-
-         if( $params{code} and $dollarat eq "" or $params{command} and $dollarbang == 0 ) {
-            $on_finish->( $pid, $exitcode );
-         }
-         else {
-            if( $on_error ) {
-               $on_error->( $pid, $exitcode, $dollarbang, $dollarat );
-            }
-            else {
-               $on_finish->( $pid, $exitcode ); # Don't have a way to report dollarbang/dollarat
-            }
-         }
-      },
-   );
-
-   $pid = $loop->spawn_child( %subparams, 
-      setup => \@setup,
-      on_exit => sub {
-         my ( undef, $exitcode, $dollarbang, $dollarat ) = @_;
-         $mergepoint->done( "exit", [ $exitcode, $dollarbang, $dollarat ] );
-      },
-   );
-
-   return undef unless defined $pid;
-
-   # Now install the handlers
-
-   foreach my $fd ( keys %filehandles ) {
-      my ( $myfd, $childfd, $fdopts ) = @{ $filehandles{$fd} };
-
-      close( $childfd );
-
-      my $notifier;
-
-      if( exists $fdopts->{on_read} ) {
-         my $on_read = $fdopts->{on_read};
-
-         $notifier = IO::Async::Stream->new(
-            read_handle => $myfd,
-
-            on_read => $on_read,
-
-            on_closed => sub {
-               $mergepoint->done( $fd );
-            },
-         );
-      }
-      elsif( exists $fdopts->{from} ) {
-         $notifier = IO::Async::Stream->new(
-            write_handle => $myfd,
-
-            on_outgoing_empty => sub {
-               $notifier->close;
-            },
-
-            on_closed => sub {
-               $mergepoint->done( $fd );
-            },
-         );
-
-         $notifier->write( $fdopts->{from} );
-      }
-
-      $loop->add( $notifier );
-   }
-
-   return $pid;
-}
-
-=head2 $pid = $loop->run_child( %params )
-
-This creates a new child process to run the given code block or command,
-capturing its STDOUT and STDERR streams. When the process exits, the callback
-is invoked being passed the exitcode, and content of the streams.
-
-=over 8
-
-=item command => ARRAY or STRING
-
-=item code => CODE
-
-The command or code to run in the child process (as per the C<spawn> method)
-
-=item on_finish => CODE
-
-A callback function to be called when the child process exits and closed its
-STDOUT and STDERR streams. It will be invoked in the following way:
-
- $on_finish->( $pid, $exitcode, $stdout, $stderr )
-
-=item stdin => STRING
-
-Optional. String to pass in to the child process's STDIN stream.
-
-=back
-
-This function is intended mainly as an IO::Async-compatible replacement for
-the perl C<readpipe> function (`backticks`), allowing it to replace
-
-  my $output = `command here`;
-
-with
-
- $loop->run_child(
-    command => "command here", 
-    on_finish => sub {
-       my ( undef, $exitcode, $output ) = @_;
-       ...
-    }
- );
-
-=cut
-
-sub run_child
-{
-   my $self = shift;
-   my %params = @_;
-
-   my $on_finish = delete $params{on_finish};
-   ref $on_finish eq "CODE" or croak "Expected 'on_finish' to be a CODE ref";
-
-   my $child_out;
-   my $child_err;
-
-   my %subparams;
-
-   if( my $child_stdin = delete $params{stdin} ) {
-      ref $child_stdin and croak "Expected 'stdin' not to be a reference";
-      $subparams{stdin} = { from => $child_stdin };
-   }
-
-   $subparams{code}    = delete $params{code};
-   $subparams{command} = delete $params{command};
-
-   croak "Unrecognised parameters " . join( ", ", keys %params ) if keys %params;
-
-   my $loop = $self->{loop};
-   $loop->open_child(
-      %subparams,
-      stdout => {
-         on_read => sub { 
-            my ( $stream, $buffref, $closed ) = @_;
-            $child_out = $$buffref if $closed;
-            return 0;
-         }
-      },
-
-      stderr => { 
-         on_read => sub {
-            my ( $stream, $buffref, $closed ) = @_;
-            $child_err = $$buffref if $closed;
-            return 0;
-         }
-      },
-
-      on_finish => sub {
-         my ( $kid, $exitcode ) = @_;
-         $on_finish->( $kid, $exitcode, $child_out, $child_err );
-      },
-   );
-}
-
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
-Paul Evans E<lt>leonerd@leonerd.org.ukE<gt>
+Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
