@@ -1,27 +1,25 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007,2008 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::Test;
 
 use strict;
+use warnings;
 
-our $VERSION = '0.16.001';
+our $VERSION = '0.46_001';
 
-use Exporter;
-our @ISA = qw( Exporter );
+use Exporter 'import';
 our @EXPORT = qw(
    testing_loop
    wait_for
    wait_for_stream
 );
 
-use IO::Async::Stream;
-
 =head1 NAME
 
-C<IO::Async::Test> - Utility functions for use in test scripts
+C<IO::Async::Test> - utility functions for use in test scripts
 
 =head1 SYNOPSIS
 
@@ -29,7 +27,7 @@ C<IO::Async::Test> - Utility functions for use in test scripts
  use IO::Async::Test;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new();
+ my $loop = IO::Async::Loop->new;
  testing_loop( $loop );
 
  my $result;
@@ -66,14 +64,14 @@ actions to perform, interspersed with assertions which check for given
 conditions. This goes against the very nature of C<IO::Async> which, being an
 asynchronisation framework, does not provide a linear stepped way of working.
 
-In order to write a test, the C<wait_for()> function provides a way of
+In order to write a test, the C<wait_for> function provides a way of
 synchronising the code, so that a given condition is known to hold, which
 would typically signify that some event has occured, the outcome of which can
 now be tested using the usual testing primitives.
 
 Because the primary purpose of C<IO::Async> is to provide IO operations on
 filehandles, a great many tests will likely be based around connected pipes or
-socket handles. The C<wait_for_stream()> function provides a convenient way
+socket handles. The C<wait_for_stream> function provides a convenient way
 to wait for some content to be written through such a connected stream.
 
 =cut
@@ -86,7 +84,7 @@ my $loop;
 
 =head2 testing_loop( $loop )
 
-Set the C<IO::Async::Loop> object which the C<wait_for()> function will loop
+Set the C<IO::Async::Loop> object which the C<wait_for> function will loop
 on.
 
 =cut
@@ -98,8 +96,8 @@ sub testing_loop
 
 =head2 wait_for( $condfunc )
 
-Repeatedly call the C<loop_once()> method on the underlying loop (given to the
-C<testing_loop()> function), until the given condition function callback
+Repeatedly call the C<loop_once> method on the underlying loop (given to the
+C<testing_loop> function), until the given condition function callback
 returns true.
 
 To guard against stalled scripts, if the loop indicates a timeout for 10
@@ -111,58 +109,76 @@ sub wait_for(&)
 {
    my ( $cond ) = @_;
 
-   my ( undef, $callerfile, $callerline ) = caller();
+   my ( undef, $callerfile, $callerline ) = caller;
 
-   while( !$cond->() ) {
-      my $retries = 10; # Give code a generous 10 seconds to do something
-      while( $retries-- ) {
-         my $subcount = $loop->loop_once( 1 );
-         last if $subcount;
+   my $timedout = 0;
+   my $timerid = $loop->enqueue_timer(
+      delay => 10,
+      code => sub { $timedout = 1 },
+   );
 
-         die "Nothing was ready after 10 second wait; called at $callerfile line $callerline\n" if $retries == 0;
-      }
+   $loop->loop_once( 1 ) while !$cond->() and !$timedout;
+
+   if( $timedout ) {
+      die "Nothing was ready after 10 second wait; called at $callerfile line $callerline\n";
+   }
+   else {
+      $loop->cancel_timer( $timerid );
    }
 }
 
 =head2 wait_for_stream( $condfunc, $handle, $buffer )
 
-Set up an C<IO::Async::Stream> object around the given $handle. Data read from
-the stream will be appended into $buffer (which is NOT initialised when the
-function is entered, in case data remains from a previous call). The
-C<loop_once> method is then repeatedly called until the condition function
-callback returns true. After this, the temporary stream will be removed from
-the loop.
+As C<wait_for>, but will also watch the given IO handle for readability, and
+whenever it is readable will read bytes in from it into the given buffer. The
+buffer is NOT initialised when the function is entered, in case data remains
+from a previous call.
+
+C<$buffer> can also be a CODE reference, in which case it will be invoked
+being passed data read from the handle, whenever it is readable.
 
 =cut
 
 sub wait_for_stream(&$$)
 {
-   my ( $cond, $handle, $DUMMY ) = @_;
-   my $varref = \$_[2]; # So that we can modify it from the on_read callback
+   my ( $cond, $handle, undef ) = @_;
 
-   my $stream = IO::Async::Stream->new(
-      read_handle => $handle,
+   my $on_read;
+   if( ref $_[2] eq "CODE" ) {
+      $on_read = $_[2];
+   }
+   else {
+      my $varref = \$_[2];
+      $on_read = sub { $$varref .= $_[0] };
+   }
 
-      on_read => sub {
-         $$varref .= ${$_[1]};
-         ${$_[1]} = "";
-         return 0;
+   $loop->watch_io(
+      handle => $handle,
+      on_read_ready => sub {
+         my $ret = $handle->sysread( my $buffer, 8192 );
+         if( !defined $ret ) {
+            die "Read failed on $handle - $!\n";
+         }
+         elsif( $ret == 0 ) {
+            die "Read returned EOF on $handle\n";
+         }
+         $on_read->( $buffer );
       }
    );
 
-   $loop->add( $stream );
-
    # Have to defeat the prototype... grr I hate these
    &wait_for( $cond );
 
-   $loop->remove( $stream );
+   $loop->unwatch_io(
+      handle => $handle,
+      on_read_ready => 1,
+   );
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
-Paul Evans E<lt>leonerd@leonerd.org.ukE<gt>
+Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
